/**
 ****************************************************************************************
 *
 * @file led_switch.h
 *
 * @brief Header file - LED Switch Service Server.
 *
 * Copyright (C) RivieraWaves 2009-2016
 *
 *
 ****************************************************************************************
 */

#ifndef LED_SWITCH_H_
#define LED_SWITCH_H_

/**
 ****************************************************************************************
 * @addtogroup LED Switch Service Server
 * @ingroup LED_SWITCH
 * @brief LED Switch Service Server
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include "rwip_config.h"
#if (BLE_LED_SWITCH_SERVER)
#include "prf_types.h"
#include "prf.h"

/*
 * DEFINES
 ****************************************************************************************
 */


#define LED_SWITCH_IDX_MAX        (1)

/*
 * ENUMERATIONS
 ****************************************************************************************
 */

enum
{
	ATT_LED_SWITCH_SVC_UUID                = ATT_UUID_16(0xA000),	  
	ATT_LED_SWITCH_CHAR_UUID               = ATT_UUID_16(0xA001),		
};

#define LED_ATT_BT_UUID_128( uuid )      {0x9e, 0x5d, 0xc4, 0xa0, 0x3b, 0x19, 0x11, 0xee, \
                                          0xbe, 0x56, 0x02, 0x42, 0xac, 0x12, LO_UINT16( uuid ), HI_UINT16( uuid )}
/// Possible states of the CUSTOM task
enum
{
    /// Idle state
    LED_SWITCH_IDLE,
    /// Busy state
    LED_SWITCH_BUSY,
    /// Number of defined states.
    LED_SWITCH_STATE_MAX
};

/// CUSTOM Attributes database handle list
enum led_switch_att_db_handles
{
    LED_SWITCH_IDX_SVC,
    LED_SWTICH_IDX_CHAR,
    LED_SWTICH_IDX_VAL,  
    LED_SWTICH_IDX_NB,
};

///Attribute Table Indexes
enum led_switch_info
{
    // LED is light on or off
    LED_ON_OFF_STATUS,
    LED_SWITCH_CHAR_MAX,
};

///Database Configuration Flags
enum led_switch_features
{
    ///All features are supported
    LED_SWITCH_ALL_FEAT_SUP                     = 0x01FF,
};


/// Value element
struct led_switch_val_elmt
{
    /// list element header
    struct co_list_hdr hdr;
    /// value identifier
    uint8_t value;
    /// value length
    uint8_t length;
    /// value data
    uint8_t data[__ARRAY_EMPTY];
};

///Device Information Service Server Environment Variable
struct led_switch_env_tag
{
    /// profile environment
    prf_env_t prf_env;
    /// List of values set by application
    struct co_list values;
    /// Service Attribute Start Handle
    uint16_t start_hdl;
    /// Services features
    uint16_t features;
    /// Last requested value
    uint8_t  req_val;
    /// Last connection index which request value
    uint8_t  req_conidx;
    /// LED_SWITCH task state
    ke_state_t state[LED_SWITCH_IDX_MAX];
};

/*
 * GLOBAL VARIABLE DECLARATIONS
 ****************************************************************************************
 */

/*
 * FUNCTION DECLARATIONS
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Retrieve CUSTOM service profile interface
 *
 * @return CUSTOM service profile interface
 ****************************************************************************************
 */
const struct prf_task_cbs* led_switch_prf_itf_get(void);


/**
 ****************************************************************************************
 * @brief Check if an attribute shall be added or not in the database
 *
 * @param features CUSTOM features
 *
 * @return Feature config flag
 ****************************************************************************************
 */
uint32_t led_switch_compute_cfg_flag(uint16_t features);

/**
 ****************************************************************************************
 * @brief Check if the provided value length matches characteristic requirements
 * @param char_code Characteristic Code
 * @param val_len   Length of the Characteristic value
 *
 * @return status if value length is ok or not
 ****************************************************************************************
 */
uint8_t led_switch_check_val_len(uint8_t char_code, uint8_t val_len);

/**
 ****************************************************************************************
 * @brief Retrieve handle attribute from value
 *
 * @param[in] env   Service environment variable
 * @param[in] value Value to search
 *
 * @return Handle attribute from value
 ****************************************************************************************
 */
uint16_t led_switch_value_to_handle(struct led_switch_env_tag* env, uint8_t value);

/**
 ****************************************************************************************
 * @brief Retrieve value from attribute handle
 *
 * @param[in] env    Service environment variable
 * @param[in] handle Attribute handle to search
 *
 * @return  Value from attribute handle
 ****************************************************************************************
 */
uint8_t led_switch_handle_to_value(struct led_switch_env_tag* env, uint16_t handle);

/*
 * TASK DESCRIPTOR DECLARATIONS
 ****************************************************************************************
 */
extern const struct ke_state_handler led_switch_default_handler;

#endif //BLE_LED_SWITCH_SERVER

/// @} LED_SWITCH

#endif // LED_SWITCH_H_
